from __future__ import unicode_literals
from django import template
from ..conf import conf
from .base import BaseJSTemplateNode, jstemplate_tag_helper



register = template.Library()



class HandlebarsJSNode(BaseJSTemplateNode):
    def generate_node_text(self, resolved_name, file_content):
        output = (
            '<script type="text/x-handlebars-template" id="{name}">'
                '{content}'
            '</script>'
        )

        # If there are additional arguments, process the template further
        if self.args:
            output += (
                '<script>'
                    '(function(H) {{'
                        'var source = document.getElementById("{name}").innerHTML,'
                            'template = source;'
            )

            if 'precompile' in self.args:
                output += ('template = H.compile(source);'
                           'H.templates = H.templates || {{}};'
                           'H.templates["{name}"] = template;')

            if 'register_partials' in self.args:
                output += 'H.registerPartial("{name}", template);'

            output += (
                    '}})(Handlebars);'
                '</script>'
            )

        return output.format(name=resolved_name, content=file_content)



@register.tag
def handlebarsjs(parser, token):
    """
    Finds the Handlebars template for the given name and renders it surrounded
    by the requisite Handlebars <script> tags.

    We don't use the jstemplate_tag_helper here, since we can take an 
    additional parameter denoting whether to register partials inline.

    """
    return jstemplate_tag_helper('handlebarsjs', HandlebarsJSNode,
                                 parser, token)



from django import template
from .base import BaseJSTemplateNode, jstemplate_tag_helper


register = template.Library()


class DustJSNode(BaseJSTemplateNode):
    def generate_node_text(self, resolved_name, file_content):
        output = file_content
        output = output.replace('\\', r'\\')
        output = output.replace('\n', r'\n')
        output = output.replace("'", r"\'")

        output = (
            '<script type="text/javascript">'
            "if (typeof(dust) !== 'undefined') {"
                "compiled = dust.compile('%s', '%s');"
                "dust.loadSource(compiled);"
            "}"
            '</script>'
        ) % (output, format(resolved_name))
        return output

@register.tag
def dustjs(parser, token):
    """
    Finds the DustJS template for the given name and compiles.

    """
    return jstemplate_tag_helper('dustjs', DustJSNode,
                                 parser, token)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals
import mock
import os.path

from django.template import Template, Context, TemplateSyntaxError
from django.test import TestCase
from mock import patch
import six

from .utils import override_settings



__all__ = [
    "MustacheJSTemplateTagTest",
    "RawTemplateTagTest",
    "ICHTemplateTagTest",
    "HandlebarsJSTemplateTagTest",
    "DustTemplateTagTest",
]


DIR = os.path.join(os.path.dirname(__file__), "templates")


class BaseJSTemplateTagTestMixin (object):
    # The following should be true for all tag types.  Remember to define
    # tag_string for any derived test classes.

    @override_settings(JSTEMPLATE_DIRS=[DIR], DEBUG=False)
    def test_no_template(self):
        res = Template(
            "{{% load jstemplate %}}{{% {0} 'notemplate' %}}".format(self.tag_string)
            ).render(Context())

        self.assertEqual(res, "")


    @override_settings(JSTEMPLATE_DIRS=[DIR], DEBUG=True)
    def test_multiple_templates_with_regex(self):
        res = Template(
            "{{% load jstemplate %}}{{% {0} '(many.*)' %}}".format(self.tag_string)
            ).render(Context())

        import re
        self.assertEqual(len(re.findall('Mustache #1', res)), 1)
        self.assertEqual(len(re.findall('Mustache #2', res)), 1)
        self.assertEqual(len(re.findall('Mustache #3', res)), 1)


    @override_settings(JSTEMPLATE_DIRS=[DIR], DEBUG=True)
    def test_multiple_templates_with_glob(self):
        res = Template(
            "{{% load jstemplate %}}{{% {0} 'many*' %}}".format(self.tag_string)
            ).render(Context())

        import re
        self.assertEqual(len(re.findall('Mustache #1', res)), 1)
        self.assertEqual(len(re.findall('Mustache #2', res)), 1)
        self.assertEqual(len(re.findall('Mustache #3', res)), 1)


    @override_settings(JSTEMPLATE_DIRS=[DIR], DEBUG=True)
    def test_no_template_debug(self):
        from jstemplate.loading import JSTemplateNotFound
        with self.assertRaises(JSTemplateNotFound):
            Template(
                "{{% load jstemplate %}}{{% {0} 'notemplate' %}}".format(self.tag_string)
                ).render(Context())


    @override_settings(JSTEMPLATE_DIRS=[DIR])
    def test_no_break_out(self):
        res = Template(
                "{{% load jstemplate %}}{{% {0} '../outside_dir' %}}".format(self.tag_string)
                ).render(Context())

        self.assertEqual(res, "")


    @override_settings(JSTEMPLATE_DIRS=[DIR])
    def test_no_absolute(self):
        res = Template(
                "{{% load jstemplate %}}{{% {0} '/testtemplate' %}}".format(self.tag_string)
                ).render(Context())

        self.assertEqual(res, "")


    def test_bad_args(self):
        with self.assertRaises(TemplateSyntaxError):
            Template(
                "{{% load jstemplate %}}{{% {0} %}}".format(self.tag_string)
                ).render(Context())

#    @override_settings(JSTEMPLATE_DIRS=[DIR])
#    def test_calls_preprocessors(self):
#        from jstemplate.tests.mockpreprocessors import MockPreprocessor1
#        from jstemplate.tests.mockpreprocessors import MockPreprocessor2
#        p1 = MockPreprocessor1()
#        p2 = MockPreprocessor2()
#        p1.process = mock.Mock(return_value="duck")
#        p2.process = mock.Mock(return_value="duck")

#        with patch('jstemplate.loading.preprocessors', [p1,p2]):
#            res = Template(
#                      "{{% load jstemplate %}}{{% {0} '/testtemplate' %}}".format(self.tag_string)
#                      ).render(Context())

#            self.assertEqual(p1.process.call_count, 1)
#            self.assertEqual(p2.process.call_count, 1)

class MustacheJSTemplateTagTest(TestCase, BaseJSTemplateTagTestMixin):
    tag_string = 'mustachejs'

    @override_settings(JSTEMPLATE_DIRS=[DIR])
    def test_unicode(self):
        res = Template(
            "{% load jstemplate %}{% mustachejs 'unicodetemplate' %}"
            ).render(Context())

        expected = (
            "<script>Mustache.TEMPLATES=Mustache.TEMPLATES||{};"
            "Mustache.TEMPLATES['unicodetemplate']="
            "'北京';"
            "</script>")

        self.assertEqual(res, expected)


    @override_settings(JSTEMPLATE_DIRS=[DIR])
    def test_simple(self):
        res = Template(
            "{% load jstemplate %}{% mustachejs 'testtemplate' %}"
            ).render(Context())

        self.assertEqual(
            res,
             "<script>Mustache.TEMPLATES=Mustache.TEMPLATES||{};"
             "Mustache.TEMPLATES['testtemplate']="
            r"'<p>Mustache\'s template full of {{ foo }} and \\.</p>\n';"
             "</script>")


    @override_settings(JSTEMPLATE_DIRS=[DIR])
    def test_variable_template_name(self):
        res = Template(
            "{% load jstemplate %}{% mustachejs templatename %}").render(
            Context({"templatename": "testtemplate"}))

        self.assertEqual(
            res,
             "<script>Mustache.TEMPLATES=Mustache.TEMPLATES||{};"
             "Mustache.TEMPLATES['testtemplate']="
            r"'<p>Mustache\'s template full of {{ foo }} and \\.</p>\n';"
             "</script>")




class ICHTemplateTagTest(TestCase, BaseJSTemplateTagTestMixin):
    tag_string = 'icanhazjs'

    @override_settings(JSTEMPLATE_DIRS=[DIR])
    def test_unicode(self):
        res = Template(
            "{% load jstemplate %}{% icanhazjs 'unicodetemplate' %}"
            ).render(Context())

        expected = (
            '<script type="text/html" id="unicodetemplate">'
            '北京'
            '</script>')

        self.assertEqual(res, expected)


    @override_settings(JSTEMPLATE_DIRS=[DIR])
    def test_simple(self):
        res = Template(
            "{% load jstemplate %}{% icanhazjs 'testtemplate' %}"
            ).render(Context())

        self.assertEqual(
            res,
             '<script type="text/html" id="testtemplate">'
             "<p>Mustache's template full of {{ foo }} and \\.</p>\n"
             '</script>')


    @override_settings(JSTEMPLATE_DIRS=[DIR])
    def test_variable_template_name(self):
        res = Template(
            "{% load jstemplate %}{% icanhazjs templatename %}").render(
            Context({"templatename": "testtemplate"}))

        self.assertEqual(
            res,
             '<script type="text/html" id="testtemplate">'
             "<p>Mustache's template full of {{ foo }} and \\.</p>\n"
             '</script>')


class HandlebarsJSTemplateTagTest(TestCase, BaseJSTemplateTagTestMixin):
    tag_string = 'handlebarsjs'

    @override_settings(JSTEMPLATE_DIRS=[DIR])
    def test_unicode(self):
        res = Template(
            "{% load jstemplate %}{% handlebarsjs 'unicodetemplate' %}"
            ).render(Context())

        expected = (
            '<script type="text/x-handlebars-template" id="unicodetemplate">'
            '北京'
            '</script>')

        self.assertEqual(res, expected)


    @override_settings(JSTEMPLATE_DIRS=[DIR])
    def test_simple(self):
        res = Template(
            "{% load jstemplate %}{% handlebarsjs 'testtemplate' %}"
            ).render(Context())

        self.assertEqual(
            res,
             '<script type="text/x-handlebars-template" id="testtemplate">'
             "<p>Mustache's template full of {{ foo }} and \\.</p>\n"
             '</script>')


    @override_settings(JSTEMPLATE_DIRS=[DIR])
    def test_variable_template_name(self):
        res = Template(
            "{% load jstemplate %}{% handlebarsjs templatename %}").render(
            Context({"templatename": "testtemplate"}))

        self.assertEqual(
            res,
             '<script type="text/x-handlebars-template" id="testtemplate">'
             "<p>Mustache's template full of {{ foo }} and \\.</p>\n"
             '</script>')


    @override_settings(JSTEMPLATE_DIRS=[DIR])
    def test_with_precompile(self):
        res = Template(
            "{% load jstemplate %}{% handlebarsjs 'testtemplate' precompile %}"
            ).render(Context())

        self.assertEqual(
            res,
             '<script type="text/x-handlebars-template" id="testtemplate">'
             '<p>Mustache\'s template full of {{ foo }} and \\.</p>\n'
             '</script><script>(function(H) {var source = document.getElementById("testtemplate").innerHTML,'
             'template = source;'
             'template = H.compile(source);'
             'H.templates = H.templates || {};'
             'H.templates["testtemplate"] = template;})(Handlebars);</script>')


    @override_settings(JSTEMPLATE_DIRS=[DIR])
    def test_with_partials(self):
        res = Template(
            "{% load jstemplate %}{% handlebarsjs 'testtemplate' register_partials %}"
            ).render(Context())

        self.assertEqual(
            res,
             '<script type="text/x-handlebars-template" id="testtemplate">'
             '<p>Mustache\'s template full of {{ foo }} and \\.</p>\n'
             '</script><script>(function(H) {var source = document.getElementById("testtemplate").innerHTML,'
             'template = source;'
             'H.registerPartial("testtemplate", template);})(Handlebars);</script>')


class DustTemplateTagTest(TestCase, BaseJSTemplateTagTestMixin):
    tag_string = 'dustjs'

    @override_settings(JSTEMPLATE_DIRS=[DIR])
    def test_unicode(self):
        res = Template(
            "{% load jstemplate %}{% dustjs 'unicodetemplate' %}"
            ).render(Context())

        expected = (
            '<script type="text/javascript">'
            "if (typeof(dust) !== 'undefined') {"
                "compiled = dust.compile('北京', 'unicodetemplate');"
                "dust.loadSource(compiled);"
            "}"
            '</script>')

        self.assertEqual(res, expected)


    @override_settings(JSTEMPLATE_DIRS=[DIR])
    def test_simple(self):
        res = Template(
            "{% load jstemplate %}{% dustjs 'testtemplate' %}"
            ).render(Context())

        self.assertEqual(
            res,
            '<script type="text/javascript">'
            "if (typeof(dust) !== 'undefined') {"
                "compiled = dust.compile('<p>Mustache\\'s template full of {{ foo }} and \\\\.</p>\\n', 'testtemplate');"
                "dust.loadSource(compiled);"
            "}"
            '</script>')


    @override_settings(JSTEMPLATE_DIRS=[DIR])
    def test_variable_template_name(self):
        res = Template(
            "{% load jstemplate %}{% dustjs templatename %}").render(
            Context({"templatename": "testtemplate"}))

        self.assertEqual(
            res,
            '<script type="text/javascript">'
            "if (typeof(dust) !== 'undefined') {"
                "compiled = dust.compile('<p>Mustache\\'s template full of {{ foo }} and \\\\.</p>\\n', 'testtemplate');"
                "dust.loadSource(compiled);"
            "}"
            '</script>')


class RawTemplateTagTest(TestCase, BaseJSTemplateTagTestMixin):
    tag_string = 'rawjstemplate'

    @override_settings(JSTEMPLATE_DIRS=[DIR])
    def test_unicode(self):
        res = Template(
            "{% load jstemplate %}{% rawjstemplate 'unicodetemplate' %}"
            ).render(Context())

        self.assertEqual(res, '北京')


    @override_settings(JSTEMPLATE_DIRS=[DIR])
    def test_simple(self):
        res = Template(
            "{% load jstemplate %}{% rawjstemplate 'testtemplate' %}"
            ).render(Context())

        self.assertEqual(
            res,
            "<p>Mustache's template full of {{ foo }} and \\.</p>\n")


    @override_settings(JSTEMPLATE_DIRS=[DIR])
    def test_variable_template_name(self):
        res = Template(
            "{% load jstemplate %}{% rawjstemplate templatename %}").render(
            Context({"templatename": "testtemplate"}))

        self.assertEqual(
            res,
            "<p>Mustache's template full of {{ foo }} and \\.</p>\n")



from django import template

from .rawjstemplate import rawjstemplate
from .handlebarsjs import handlebarsjs
from .mustachejs import mustachejs
from .icanhazjs import icanhazjs
from .dustjs import dustjs

register = template.Library()


register.tag(rawjstemplate)
register.tag(handlebarsjs)
register.tag(mustachejs)
register.tag(icanhazjs)
register.tag(dustjs)



from os.path import join, dirname

from setuptools import setup, find_packages

here = dirname(__file__)

long_description = (open(join(here, "README.rst")).read() + "\n\n" +
                    open(join(here, "CHANGES.rst")).read() + "\n\n" +
                    open(join(here, "TODO.rst")).read())

def get_version():
    fh = open(join(here, "jstemplate", "__init__.py"))
    try:
        for line in fh.readlines():
            if line.startswith("__version__ ="):
                return line.split("=")[1].strip().strip('"')
    finally:
        fh.close()

setup(
    name="django-jstemplate",
    version=get_version(),
    description="A Django template tag for embedding Mustache.js templates -- or other JavaScript templates -- safely.",
    long_description=long_description,
    author="Mjumbe Wawatu Ukweli",
    author_email="mjumbewu@gmail.com",
    url="https://github.com/mjumbewu/django-jstemplate/",
    packages=find_packages(),
    package_data={'jstemplate': ['static/libs/*.js']},
    install_requires=[
        'Django >= 1.3',
        'six'
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Framework :: Django",
    ],
    zip_safe=False,
    tests_require=["Django>=1.3", "mock", "six"],
    test_suite="runtests.runtests"
)



from __future__ import unicode_literals

import os.path
import six

from django.test import TestCase
from django.utils import translation
from jstemplate.preprocessors import I18nPreprocessor

from .utils import override_settings



__all__ = [
    "I18nTest",
]


DIR = os.path.join(os.path.dirname(__file__), "templates")


class I18nTest (TestCase):
    @override_settings(USE_I18N=True)
    def setUp(self):
        # A decorator function that just adds 'XXX ' to the front of all strings
        def wrap_with_xxx(func):
            def new_func(*args, **kwargs):
                output = func(*args, **kwargs)
                return "XXX "+output
            return new_func

        self.native_lang = translation.get_language()
        # Activate french, so that if the fr files haven't been loaded, they will be loaded now.
        translation.activate("fr")
        fake_translation = translation.trans_real._active.value

        # wrap the ugettext and ungettext functions so that 'XXX ' will prefix each translation
        if six.PY3:
            self.original_gettext = fake_translation.gettext
            fake_translation.gettext = wrap_with_xxx(fake_translation.gettext)
        else:
            self.original_ugettext = fake_translation.ugettext
            fake_translation.ugettext = wrap_with_xxx(fake_translation.ugettext)

        # Turn back on our old translations
        translation.activate(self.native_lang)

    def tearDown(self):
        # Restore the french translation function
        translation.activate("fr")
        fake_translation = translation.trans_real._active.value
        if six.PY3:
            fake_translation.gettext = self.original_gettext
        else:
            fake_translation.ugettext = self.original_ugettext

        # Turn back on our old translations
        translation.activate(self.native_lang)

    def test_no_string(self):
        res = I18nPreprocessor().process(
            '<div>{{# _ }}{{/ _ }}</div>'
        )

        self.assertEqual(res, '<div></div>')

    def test_short_style_simple_string_translation(self):
        translation.activate('fr')
        res = I18nPreprocessor().process(
            '<div>{{ _ "Hello, world!" }}</div>'
        )

        self.assertEqual(res, '<div>{{ "XXX Hello, world!" }}</div>')

    def test_short_style_no_string(self):
        res = I18nPreprocessor().process(
            '<div>{{_ ""}}</div>'
        )

        self.assertEqual(res, '<div>{{""}}</div>')

    def test_short_style_ignore_mismatched_quotes(self):
        res = I18nPreprocessor().process(
            '<div>{{_ "\'}}</div>'
        )

        self.assertEqual(res, '<div>{{_ "\'}}</div>')

    @override_settings(JSTEMPLATE_I18N_TAGS=['hello', 'world'])
    def test_short_style_different_tagname(self):
        res = I18nPreprocessor().process(
            "<div>{{ hello 'World' }}</div>"
        )

        self.assertEqual(res, "<div>{{ 'World' }}</div>")

        res = I18nPreprocessor().process(
            '<div>{{ _ "World" }}</div>'
        )

        self.assertEqual(res, '<div>{{ _ "World" }}</div>')

    @override_settings(JSTEMPLATE_I18N_TAGS=['hello', 'world'])
    def test_different_tagname(self):
        res = I18nPreprocessor().process(
            '<div>{{# hello }}{{name}}{{/ hello }}</div>'
        )

        self.assertEqual(res, '<div>{{name}}</div>')

        res = I18nPreprocessor().process(
            '<div>{{# _ }}{{name}}{{/ _ }}</div>'
        )

        self.assertEqual(res, '<div>{{# _ }}{{name}}{{/ _ }}</div>')

    def test_simple_string_translation(self):
        translation.activate('fr')
        res = I18nPreprocessor().process(
            '<div>{{# _ }}Hello, {{name}}!{{/ _ }}</div>'
        )

        self.assertEqual(res, '<div>XXX Hello, {{name}}!</div>')

    def test_string_with_newlines_translation(self):
        translation.activate('fr')
        res = I18nPreprocessor().process(
            '<div>{{# _ }}Hello, \n{{name}}\n!{{/ _ }}</div>'
        )

        self.assertEqual(res, '<div>XXX Hello, \n{{name}}\n!</div>')

    def test_string_with_newlines_translation(self):
        translation.activate('fr')
        res = I18nPreprocessor().process(
            '<div>{{# _ }}Hello, \n{{name}}\n!{{/ _ }} {{# _ }}Second string{{/ _ }}</div>'
        )

        self.assertEqual(res, '<div>XXX Hello, \n{{name}}\n! XXX Second string</div>')



from __future__ import unicode_literals

import re
from django.utils.translation import ugettext
from .conf import conf

class I18nPreprocessor(object):
    @property
    def tagnames(self):
        return conf.JSTEMPLATE_I18N_TAGS

    @property
    def short_trans_re(self):
        # Should match strings like: {{ _ "Hello, world! }}
        tagnames = '|'.join(['(?:{0})'.format(t) for t in self.tagnames])
        
        left_side = r'''(?P<left>\{\{\s*(?P<tag>(?:''' + tagnames + r''')\s+)(?P<quote>['"]))'''
        right_side = r'''(?P<right>(?P=quote)\s*\}\})'''

        return left_side + r'(?P<msg>.*?)' + right_side

    @property
    def long_trans_re(self):
        # Should match strings like: {{# _ }}Hello, {{ name }}.{{/ _ }}
        tagnames = '|'.join(['(?:{0})'.format(t) for t in self.tagnames])
        
        start_tag = r'\{\{#\s*(?P<tag>' + tagnames + r')\s*\}\}'
        end_tag = r'\{\{\/\s*(?P=tag)\s*\}\}'

        return start_tag + r'(?P<msg>.*?)' + end_tag

    def translate_short_form(self, match):
        """Translate a result of matching the compiled trans_re pattern."""
        tag = match.group('tag')
        msg = match.group('msg')
        msg = ugettext(msg) if len(msg) > 0 else ''
        string = match.group('left').replace(tag, '', 1) + msg + match.group('right')
        return string

    def translate_long_form(self, match):
        """Translate a result of matching the compiled trans_re pattern."""
        msg = match.group('msg')
        string = ugettext(msg) if len(msg) > 0 else ''
        return string

    def process(self, content):
        # TODO: cache the compiled regex.
        # We need to compile here because re.sub doesn't accept a flags
        # argument in python < 2.7, but re.compile does.
        pattern = re.compile(self.short_trans_re)
        content = re.sub(pattern, self.translate_short_form, content)

        pattern = re.compile(self.long_trans_re, flags=re.DOTALL)
        content = re.sub(pattern, self.translate_long_form, content)

        return content



"""
Testing utilities backported from recent Django versions, for testing with
older Django versions.

"""
from __future__ import with_statement

from django.conf import settings, UserSettingsHolder
from django.utils.functional import wraps



class override_settings(object):
    """
    Acts as either a decorator, or a context manager. If it's a decorator it
    takes a function and returns a wrapped function. If it's a contextmanager
    it's used with the ``with`` statement. In either event entering/exiting
    are called before and after, respectively, the function/block is executed.

    """
    def __init__(self, **kwargs):
        self.options = kwargs
        self.wrapped = settings._wrapped

    def __enter__(self):
        self.enable()

    def __exit__(self, exc_type, exc_value, traceback):
        self.disable()

    def __call__(self, func):
        @wraps(func)
        def inner(*args, **kwargs):
            with self:
                return func(*args, **kwargs)
        return inner

    def enable(self):
        override = UserSettingsHolder(settings._wrapped)
        for key, new_value in self.options.items():
            setattr(override, key, new_value)
        settings._wrapped = override

    def disable(self):
        settings._wrapped = self.wrapped
